from typing import Any, List, Tuple
from .enums import Operator


class Condition:
    def __init__(self, field: str, value: Any, operator: Operator = Operator.EQ):
        self.field: str = field
        self.value: Any = value
        self.operator: Operator = operator

    def parse(self) -> tuple[str, Any]:
        return f"{self.field} {self.operator.value} ?", self.value


class ConditionTree:
    def __init__(self, logic="and"):
        self.conditions: List[Condition | ConditionTree] = []
        self.logic = logic

    def count(self):
        return len(self.conditions)

    def add_condition(self, condition: Condition):
        self.conditions.append(condition)
        return self

    def add_tree(self, condition_tree: "ConditionTree"):
        self.conditions.append(condition_tree)
        return self

    def parse(self) -> Tuple[str, Tuple[Any, ...]]:
        if len(self.conditions) == 0:
            return "", ()
        args: List[Any] = []
        exps: List[str] = []
        for condition in self.conditions:
            if isinstance(condition, ConditionTree):
                exp, arg = condition.parse()
                exps.append(f"({exp})")
                args.extend(arg)
            else:
                exp, arg = condition.parse()
                exps.append(exp)
                args.append(arg)
        return f" {self.logic} ".join(exps), tuple(args)



from enum import Enum


class Middleware(Enum):
    BEFORE_QUERY = "before_query"
    BEFORE_INSERT = "before_insert"

    def __str__(self):
        return self.value

    def __repr__(self):
        return f"Middleware.{self.name}"



from typing import Any, Generic, List, Type, TypeVar, get_type_hints
from pydorm._where import Or, Where
from .protocols import EntityProtocol

T = TypeVar("T", bound=EntityProtocol)


class QueryWrapper(Generic[T]):
    def __init__(
        self,
        entity_type: Type[T],
    ):

        self._entity_type = entity_type
        self._table = entity_type.__table_name__

        self._where = Where()
        self._select_fields: List[str] = []
        self._ignore_fields: List[str] = []
        self._order_by = None
        self._limit = None
        self._offset = None
        self._distinct = False

        fields = list(get_type_hints(entity_type).keys())
        self._fields = [field for field in fields if not field.startswith("__")]

    def get_type(self) -> Type[T]:
        return self._entity_type

    def select(self, *select_fields: str, distinct: bool = False) -> "QueryWrapper[T]":
        for select_field in select_fields:
            self.check_field(select_field)

        self._select_fields = list(select_fields)
        self._distinct = distinct
        return self

    def ignore(self, *ignore_fields: str) -> "QueryWrapper[T]":
        for ignore_field in ignore_fields:
            self.check_field(ignore_field)

        self._ignore_fields = list(ignore_fields)
        return self

    def check_field(self, field: str):
        if not hasattr(self._entity_type, field):
            raise ValueError(f"invalid field [{field}] in entity [{self._entity_type}]")

    def eq(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.eq(field, value)
        return self

    def ne(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.ne(field, value)
        return self

    def gt(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.gt(field, value)
        return self

    def ge(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.ge(field, value)
        return self

    def lt(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.lt(field, value)
        return self

    def le(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.le(field, value)
        return self

    def in_(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.in_(field, value)
        return self

    def l_like(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.l_like(field, value)
        return self

    def r_like(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.r_like(field, value)
        return self

    def like(self, field: str, value: Any) -> "QueryWrapper[T]":
        self.check_field(field)
        self._where.like(field, value)
        return self

    def or_(self, or_: Or) -> "QueryWrapper[T]":
        self._where.or_(or_)
        return self

    def desc(self, *order_by: str) -> "QueryWrapper[T]":
        self._order_by = [f"{field} desc" for field in order_by]
        return self

    def asc(self, *order_by: str) -> "QueryWrapper[T]":
        self._order_by = [f"{field} asc" for field in order_by]
        return self

    def limit(self, limit: int) -> "QueryWrapper[T]":
        self._limit = limit
        return self

    def offset(self, offset: int) -> "QueryWrapper[T]":
        self._offset = offset
        return self

    def build_sql(self) -> tuple[str, tuple[Any, ...]]:
        if len(self._select_fields) == 0:
            self._select_fields = self._fields

        select_fields = [field for field in self._select_fields if field not in self._ignore_fields]

        sql = f'SELECT {"DISTINCT " if self._distinct and self._select_fields else ""}{",".join(select_fields)} FROM {self._table}'
        args = ()
        tree = self._where.tree()
        if len(tree.conditions) > 0:
            exp, args = tree.parse()
            sql += " WHERE " + exp
        if self._order_by is not None:
            sql += f' ORDER BY {",".join(self._order_by)}'
        if self._limit is not None:
            sql += f" LIMIT {self._limit}"
        if self._offset is not None:
            sql += f" OFFSET {self._offset}"

        return sql, args

    def build_count_sql(self) -> tuple[str, tuple[Any, ...]]:
        sql = f"SELECT COUNT(*) FROM {self._table}"
        args = ()
        tree = self._where.tree()
        if len(tree.conditions) > 0:
            exp, args = tree.parse()
            sql += " WHERE " + exp
        return sql, args



from typing import Any, TypeVar

from ._condition import ConditionTree, Condition
from .enums import Operator

T = TypeVar("T")


class Where:
    def __init__(self, logic="and") -> None:
        self._condition_tree = ConditionTree(logic)

    def tree(self) -> ConditionTree:
        return self._condition_tree

    def eq(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(Condition(field, value))
        return self

    def ne(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(Condition(field, value, Operator.NE))
        return self

    def gt(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(Condition(field, value, Operator.GT))
        return self

    def ge(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(Condition(field, value, Operator.GE))
        return self

    def lt(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(Condition(field, value, Operator.LT))
        return self

    def le(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(Condition(field, value, Operator.LE))
        return self

    def in_(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(Condition(field, value, Operator.IN))
        return self

    def l_like(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(Condition(field, f"%{value}", Operator.LIKE))
        return self

    def r_like(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(Condition(field, f"{value}%", Operator.LIKE))
        return self

    def like(self, field: str, value: Any) -> "Where":
        self._condition_tree.add_condition(
            Condition(field, f"%{value}%", Operator.LIKE)
        )
        return self

    def or_(self, or_conditions: "Or") -> "Where":
        self._condition_tree.add_tree(or_conditions._condition_tree)
        return self

    def count(self):
        return self._condition_tree.count()


class Or(Where):
    def __init__(self) -> None:
        super().__init__("or")


def where():
    return Where()


def or_():
    return Or()



from dataclasses import field, make_dataclass
from dataclasses import field, make_dataclass
from typing import Dict, List, Any, Type

from loguru import logger
from pymysql.connections import Connection

from pydorm.mysql._reusable_mysql_connection import ReusableMysqlConnection
from ._mysql_executor import mysql_executor, MysqlExecutor
from ._mysql_table_inspector import mysql_table_inspector


class MysqlDataSource:
    def __init__(
        self,
        data_source_id: str,
        host: str,
        port: int,
        user: str,
        password: str,
        database: str,
        **options: Any,
    ):
        self._data_source_id: str = data_source_id
        self._dialect: str = "mysql"
        self._host: str = host
        self._port: int = port
        self._user: str = user
        self._password: str = password
        self._database: str = database
        self._reusable_connection: ReusableMysqlConnection = ReusableMysqlConnection(
            self._data_source_id,
            self.create_connection,
        )
        self._executor: MysqlExecutor = mysql_executor
        self._models: Dict[str, Type[Any]] = {}
        self._options: Dict[str, Any] = options

    def get_id(self) -> str:
        return self._data_source_id

    def get_dialect(self) -> str:
        return self._dialect

    def get_database(self) -> str:
        return self._database

    def create_connection(self) -> Connection:
        import pymysql
        from pymysql.cursors import DictCursor

        try:
            conn = pymysql.connect(
                host=self._host,
                port=self._port,
                user=self._user,
                password=self._password,  # 移除编码
                database=self._database,
                cursorclass=DictCursor,
                charset="utf8mb4",  # 添加字符集
                autocommit=False,  # 设置自动提交为False,
                **self._options,  # 传递其他选项
            )
            logger.info(f"[{self._data_source_id}] create connection [{id(conn)}]")
            return conn
        except Exception as e:
            logger.error(f"[{self._data_source_id}] Failed to create connection: {e}")
            raise

    def get_reusable_connection(self) -> ReusableMysqlConnection:
        return self._reusable_connection

    def close(self):
        """关闭数据源和相关连接"""
        if self._reusable_connection:
            self._reusable_connection.close()
            logger.info(f"[{self._data_source_id}] DataSource closed")

    def get_executor(self) -> MysqlExecutor:
        return self._executor

    def get_model(self, database: str | None, table: str) -> Type[Any]:
        key = f"{database or self._database}.{table}"
        if key not in self._models:
            table_structure: List[Dict] = mysql_table_inspector.load_structure(
                self._reusable_connection, database or self._database, table
            )
            fields = [
                (table_field["field_"], any, field(default=None)) for table_field in table_structure
            ]
            self._models[key] = make_dataclass(key, fields=fields)
        return self._models[key]

    def remove_model(self, database: str | None, table: str):
        key = f"{database or self._database}.{table}"
        if key in self._models:
            del self._models[key]
            logger.info(f"[{self._data_source_id}] Model {key} removed")
        else:
            logger.warning(f"[{self._data_source_id}] Model {key} not found")

    def load_structure(self, database: str, table: str = "") -> List[Dict[str, Any]]:
        """加载数据源的表结构"""
        if not table:
            raise ValueError("Table name must be provided to load structure")
        return mysql_table_inspector.load_structure(
            self._reusable_connection, database or self._database, table
        )



from typing import Any, Dict, List, Tuple, TypeVar

from ._middlewares import before_query_middlewares
from ._query_wrapper import QueryWrapper
from .mysql._mysql_data_source import MysqlDataSource
from .mysql._reusable_mysql_connection import ReusableMysqlConnection
from .utils.random_utils import generate_random_string

T = TypeVar("T", bound=Any)


def find(
    wrapper: QueryWrapper[T],
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
) -> T | None:
    result = find_dict(wrapper, conn, data_source)
    if result is None:
        return None
    return wrapper.get_type()(**result)


def find_dict(
    wrapper: QueryWrapper[T],
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
) -> Dict[str, Any] | None:

    if data_source is None:
        raise ValueError("data_source must be provided")

    operation_id = generate_random_string("R-", 10)

    for middleware in before_query_middlewares:
        if callable(middleware):
            middleware(wrapper)

    sql, args = wrapper.build_sql()

    if conn is None:
        new_conn = data_source.get_reusable_connection()
        try:
            new_conn.acquire(operation_id=operation_id)
            new_conn.begin()
            result: Dict[str, Any] | None = data_source.get_executor().select_one(new_conn, sql, args)
            new_conn.commit()
            return result
        finally:
            new_conn.release(operation_id=operation_id)
    else:
        return data_source.get_executor().select_one(conn, sql, args)


def list(
    wrapper: QueryWrapper[T],
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
) -> List[T]:
    result = list_dict(wrapper, conn, data_source)
    if result is None:
        return []
    return [wrapper.get_type()(**item) for item in result]


def list_dict(
    wrapper: QueryWrapper[T],
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
) -> List[Dict[str, Any]]:
    if data_source is None:
        raise ValueError("data_source must be provided")

    operation_id = generate_random_string("R-", 10)

    for middleware in before_query_middlewares:
        if callable(middleware):
            middleware(wrapper)

    sql, args = wrapper.build_sql()
    if conn is None:
        new_conn = data_source.get_reusable_connection()
        try:
            new_conn.acquire(operation_id=operation_id)
            new_conn.begin()
            result: List[Dict[str, Any]] | None = data_source.get_executor().select_many(new_conn, sql, args)
            new_conn.commit()
            return result
        finally:
            new_conn.release(operation_id=operation_id)
    else:
        return data_source.get_executor().select_many(conn, sql, args)


def count(
    wrapper: QueryWrapper[T],
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
    load_middlewares: bool = True,
) -> int:
    if data_source is None:
        raise ValueError("data_source must be provided")

    operation_id = generate_random_string("R-", 10)

    if load_middlewares:
        for middleware in before_query_middlewares:
            if callable(middleware):
                middleware(wrapper)

    sql, args = wrapper.build_count_sql()

    if conn is None:
        new_conn = data_source.get_reusable_connection()
        try:
            new_conn.acquire(operation_id=operation_id)
            new_conn.begin()
            result = data_source.get_executor().select_one(new_conn, sql, args)
            new_conn.commit()

            if result is None:
                return 0
            return result["COUNT(*)"]
        finally:
            new_conn.release(operation_id=operation_id)
    else:
        result = data_source.get_executor().select_one(conn, sql, args)
        if result is None:
            return 0
        return result["COUNT(*)"]


def page(
    wrapper: QueryWrapper[T],
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
    current: int = 1,
    page_size: int = 10,
) -> Tuple[List[T], int]:
    if data_source is None:
        raise ValueError("data_source must be provided")

    rows, total = page_dict(wrapper, conn, data_source, current, page_size)
    if rows is None:
        return [], 0
    return [wrapper.get_type()(**row) for row in rows], total


def page_dict(
    wrapper: QueryWrapper[T],
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
    current: int = 1,
    page_size: int = 10,
) -> Tuple[List[Dict[str, Any]], int]:
    if data_source is None:
        raise ValueError("data_source must be provided")

    operation_id = generate_random_string("R-", 10)

    for middleware in before_query_middlewares:
        if callable(middleware):
            middleware(wrapper)

    wrapper.limit(page_size).offset((current - 1) * page_size)
    sql, args = wrapper.build_sql()

    if conn is None:
        new_conn = data_source.get_reusable_connection()
        try:
            new_conn.acquire(operation_id=operation_id)
            new_conn.begin()
            total = count(wrapper, new_conn, data_source, load_middlewares=False)
            if total == 0:
                return [], total
            rows = data_source.get_executor().select_many(new_conn, sql, args)
            new_conn.commit()
            return rows, total
        finally:
            new_conn.release(operation_id=operation_id)

    total = count(wrapper, conn, data_source, load_middlewares=False)
    if total == 0:
        return [], total
    rows = data_source.get_executor().select_many(conn, sql, args)
    return rows, total



from typing import List, Dict

from ._reusable_mysql_connection import ReusableMysqlConnection


class MysqlTableInspector:

    # noinspection PyMethodMayBeStatic
    def load_structure(
        self, conn: ReusableMysqlConnection, database: str, table: str
    ) -> List[Dict[str, str]]:
        need_acquire = not conn.is_locked()
        if need_acquire:
            conn.acquire()
        try:
            cursor = conn.cursor()
            try:
                sql = f"show full columns from {database}.{table}"
                cursor.execute(sql)
                rows = cursor.fetchall()
                table_fields: List[Dict[str, str]] = []
                for row in rows:
                    table_field = dict(
                        field_=row["Field"],
                        type_=row["Type"],
                        null_=row["Null"],
                        key_=row["Key"],
                        default_=row["Default"],
                        extra=row["Extra"],
                        comment=row.get("Comment", ""),
                    )
                    table_fields.append(table_field)
                return table_fields
            finally:
                cursor.close()
        finally:
            if need_acquire:
                conn.release()


mysql_table_inspector = MysqlTableInspector()



from typing import Any, Dict, Generic, List, Literal, Tuple, Type, TypeVar, get_type_hints
from .protocols import EntityProtocol

T = TypeVar("T", bound=EntityProtocol)


class InsertWrapper(Generic[T]):
    def __init__(self, entity_type: Type[T]):
        self._entity_type = entity_type
        self._table = entity_type.__table_name__

        fields = list(get_type_hints(entity_type).keys())
        self._fields = [field for field in fields if not field.startswith("__")]

    def build_insert_sql(
        self, data: Dict[str, Any], duplicate_key_update: List[str] | Literal["all"] | None = None
    ) -> Tuple[str, Tuple[Any, ...]]:
        placeholder: List[str] = []
        keys: List[str] = []
        values: List[Any] = []
        for k, v in data.items():
            if k in self._fields:
                if v is None:
                    continue
                keys.append(k)
                values.append(v)
                placeholder.append("?")

        if len(keys) == 0:
            raise ValueError("no valid field found")

        sql = f'INSERT INTO {self._table}({",".join(keys)}) VALUES({",".join(placeholder)})'
        if duplicate_key_update is not None:
            if isinstance(duplicate_key_update, str) and duplicate_key_update == "all":
                sql += f' ON DUPLICATE KEY UPDATE {",".join([f"{k}=VALUES({k})" for k in keys])}'
            elif len(duplicate_key_update) > 0:
                sql += f' ON DUPLICATE KEY UPDATE {",".join([f"{k}=VALUES({k})" for k in duplicate_key_update])}'

        args = tuple(values)

        return sql, args

    def build_insert_bulk_sql(
        self,
        data: List[Dict[str, Any]],
        duplicate_key_update: List[str] | Literal["all"] | None = None,
    ) -> Tuple[str, List[Tuple[Any, ...]]]:
        placeholder: List[str] = []
        keys: List[str] = []
        for k, _ in data[0].items():
            if k in self._fields:
                keys.append(k)
                placeholder.append("?")

        if len(keys) == 0:
            raise ValueError("no valid field found")

        sql = f'INSERT INTO {self._table}({",".join(keys)}) VALUES({",".join(placeholder)})'

        if duplicate_key_update is not None:
            if isinstance(duplicate_key_update, str) and duplicate_key_update == "all":
                sql += f' ON DUPLICATE KEY UPDATE {",".join([f"{k}=VALUES({k})" for k in keys])}'
            elif len(duplicate_key_update) > 0:
                sql += f' ON DUPLICATE KEY UPDATE {",".join([f"{k}=VALUES({k})" for k in duplicate_key_update])}'

        args = [tuple(datum[k] for k in keys) for datum in data]
        return sql, args



import yaml

from ._dorm import dorm, Dorm


def init(path: str) -> Dorm:
    with open(path, 'r') as f:
        config_dict = yaml.load(f, Loader=yaml.FullLoader)
        dorm.init(config_dict)
    return dorm



from typing import Any, Generic, Type, TypeVar, get_type_hints
from pydorm._where import Or, Where
from .protocols import EntityProtocol

T = TypeVar("T", bound=EntityProtocol)


class DeleteWrapper(Generic[T]):
    def __init__(
        self,
        entity_type: Type[T],
    ):

        self._entity_type = entity_type
        self._table = entity_type.__table_name__

        self._where = Where()

        fields = list(get_type_hints(entity_type).keys())
        self._fields = [field for field in fields if not field.startswith("__")]

    def get_type(self) -> Type[T]:
        return self._entity_type

    def check_field(self, field: str):
        if not hasattr(self._entity_type, field):
            raise ValueError(f"invalid field [{field}] in entity [{self._entity_type}]")

    def eq(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.eq(field, value)
        return self

    def ne(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.ne(field, value)
        return self

    def gt(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.gt(field, value)
        return self

    def ge(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.ge(field, value)
        return self

    def lt(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.lt(field, value)
        return self

    def le(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.le(field, value)
        return self

    def in_(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.in_(field, value)
        return self

    def l_like(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.l_like(field, value)
        return self

    def r_like(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.r_like(field, value)
        return self

    def like(self, field: str, value: Any) -> "DeleteWrapper[T]":
        self.check_field(field)
        self._where.like(field, value)
        return self

    def or_(self, or_: Or) -> "DeleteWrapper[T]":
        self._where.or_(or_)
        return self

    def build_sql(self) -> tuple[str, tuple[Any, ...]]:
        sql = f"DELETE FROM {self._table}"
        args = ()

        exp, args2 = self._where.tree().parse()
        sql += " WHERE " + exp
        args += args2
        return sql, args



from typing import Any, Callable, Dict, List

from ._delete_wrapper import DeleteWrapper
from ._query_wrapper import QueryWrapper
from ._update_wrapper import UpdateWrapper

before_query_middlewares: List[
    Callable[[QueryWrapper[Any] | UpdateWrapper[Any] | DeleteWrapper[Any]], None]
] = []
before_insert_middlewares: List[Callable[[Dict[str, Any] | List[Dict[str, Any]]], None]] = []


def use_query_middleware(
    middleware: Callable[[QueryWrapper[Any] | UpdateWrapper[Any] | DeleteWrapper[Any]], None],
):
    before_query_middlewares.append(middleware)


def use_insert_middleware(middleware: Callable[[Dict[str, Any] | List[Dict[str, Any]]], None]):
    before_insert_middlewares.append(middleware)



import random
import string


def generate_random_string(prefix: str = "", length=10, seed: int | None = None) -> str:
    """
    生成一个随机的字符串，包含字母和数字。

    Args:
        prefix (str): 字符串前缀，默认为空字符串。
        length (int): 字符串长度，默认为10。
        seed (int | None): 随机种子，默认为 None。

    Returns:
        str: 随机生成的字符串。
    """
    if seed is not None:
        random.seed(seed)
    characters = string.ascii_letters + string.digits
    return prefix + "".join(random.choices(characters, k=length))



from contextlib import contextmanager
from typing import Any, Dict, List, Tuple

from loguru import logger

from ._reusable_mysql_connection import ReusableMysqlConnection


class MysqlExecutor:
    """MySQL数据库执行器，提供常用的数据库操作方法"""

    def __init__(self, log_sql: bool = True):
        """
        初始化MySQL执行器

        Args:
            log_sql: 是否记录SQL日志
        """
        self.log_sql = log_sql

    def _log_execution(self, conn: ReusableMysqlConnection, sql: str, args: Any) -> None:
        """记录SQL执行日志"""
        if not self.log_sql:
            return

        logger.debug(f"[{id(conn)}] {sql}")
        # if args is not None and len(args) > 0:
        #     logger.debug(f"### {args}")

    # noinspection PyMethodMayBeStatic
    def _prepare_sql(self, sql: str) -> str:
        """将占位符从?转换为%s"""
        return sql.replace("?", "%s")

    @contextmanager
    def _get_cursor(self, conn: ReusableMysqlConnection):
        """获取游标的上下文管理器"""
        cursor = conn.cursor()
        try:
            yield cursor
        finally:
            cursor.close()

    def select_one(
        self,
        conn: ReusableMysqlConnection,
        sql: str,
        args: Tuple[Any, ...] = (),
    ) -> Dict[str, Any] | None:
        """
        执行查询并返回单行结果

        Args:
            conn: 数据库连接
            sql: SQL语句
            args: 参数元组

        Returns:
            查询结果字典或None
        """
        self._log_execution(conn, sql, args)

        with self._get_cursor(conn) as cursor:
            prepared_sql = self._prepare_sql(sql)
            cursor.execute(prepared_sql, args)

            if cursor.rowcount == 0:
                return None
            return cursor.fetchone()

    def select_many(
        self,
        conn: ReusableMysqlConnection,
        sql: str,
        args: tuple[Any, ...] = (),
    ) -> list[dict[str, Any]]:
        """
        执行查询并返回多行结果

        Args:
            conn: 数据库连接
            sql: SQL语句
            args: 参数元组

        Returns:
            查询结果列表
        """
        self._log_execution(conn, sql, args)

        with self._get_cursor(conn) as cursor:
            prepared_sql = self._prepare_sql(sql)
            cursor.execute(prepared_sql, args)

            if cursor.rowcount == 0:
                return []

            rows = cursor.fetchall()
            return list(rows) if rows else []

    def execute(
        self,
        conn: ReusableMysqlConnection,
        sql: str,
        args: Tuple[Any, ...] = (),
    ) -> Tuple[int, int]:
        """
        执行SQL语句（INSERT、UPDATE、DELETE）

        Args:
            conn: 数据库连接
            sql: SQL语句
            args: 参数元组

        Returns:
            (受影响的行数, 最后插入的行ID)
        """
        self._log_execution(conn, sql, args)

        with self._get_cursor(conn) as cursor:
            prepared_sql = self._prepare_sql(sql)
            row_affected = cursor.execute(prepared_sql, args)
            last_row_id = cursor.lastrowid
            return row_affected, last_row_id

    def executemany(
        self,
        conn: ReusableMysqlConnection,
        sql: str,
        args: List[Tuple[Any, ...]],
    ) -> int:
        """
        批量执行SQL语句

        Args:
            conn: 数据库连接
            sql: SQL语句
            args: 参数列表

        Returns:
            受影响的总行数
        """
        self._log_execution(conn, sql, args)

        with self._get_cursor(conn) as cursor:
            prepared_sql = self._prepare_sql(sql)
            row_affected = cursor.executemany(prepared_sql, args)
            return row_affected or 0


mysql_executor = MysqlExecutor()



from typing import Any, Dict

from .mysql import MysqlDataSource


class DataSourceStorage:
    def __init__(self):
        self._data_sources: Dict[str, MysqlDataSource] = {}

    def default(self) -> MysqlDataSource | None:
        return self._data_sources.get("default", None)

    def load(self, config_dict: Dict[str, Any]):
        data_source_config = config_dict.get("data_source")
        if data_source_config is None:
            raise ValueError("data_source config not found")

        for data_source_id, conf in data_source_config.items():
            if "dialect" not in conf:
                raise ValueError("dialect is required")

            if "mysql" == conf["dialect"]:
                mysql_ds = MysqlDataSource(
                    data_source_id=data_source_id,
                    host=conf["host"],
                    port=conf["port"],
                    user=conf["user"],
                    password=conf["password"],
                    database=conf["database"],
                )
                self._data_sources[data_source_id] = mysql_ds
            else:
                raise ValueError(f'unsupported dialect: {conf["dialect"]}')

    def add_datasource(
        self,
        data_source_id: str,
        dialect: str,
        host: str,
        port: int,
        user: str,
        password: str,
        database: str,
        **options: Any,
    ):
        if dialect is None or dialect == "":
            raise ValueError("dialect is required")
        if host is None or host == "":
            raise ValueError("host is required")
        if port is None or port <= 0:
            raise ValueError("port is required and must be greater than 0")
        if user is None or user == "":
            raise ValueError("user is required")
        if password is None or password == "":
            raise ValueError("password is required")
        if database is None or database == "":
            raise ValueError("database is required")

        if data_source_id in self._data_sources:
            raise ValueError(f"data source with id {data_source_id} already exists")

        if "mysql" == dialect:
            mysql_ds = MysqlDataSource(
                data_source_id=data_source_id,
                host=host,
                port=port,
                user=user,
                password=password,
                database=database,
                **options,
            )
            self._data_sources[data_source_id] = mysql_ds
        else:
            raise ValueError(f"unsupported dialect: {dialect}")

    def get(self, data_source_id: str) -> MysqlDataSource | None:
        return self._data_sources.get(data_source_id, None)



from typing import Any, TypeVar

from ._middlewares import before_query_middlewares
from ._update_wrapper import UpdateWrapper
from .mysql._mysql_data_source import MysqlDataSource
from .mysql._reusable_mysql_connection import ReusableMysqlConnection
from .utils.random_utils import generate_random_string

T = TypeVar("T", bound=Any)


def update(
    wrapper: UpdateWrapper[T],
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
) -> int:
    if wrapper._where.count() == 0:
        raise ValueError("where condition is required for update operation")

    if len(wrapper._update_fields) == 0:
        raise ValueError("update fields are required")

    if data_source is None:
        raise ValueError("data_source must be provided")

    operation_id = generate_random_string("U-", 10)

    for middleware in before_query_middlewares:
        if callable(middleware):
            middleware(wrapper)

    sql, args = wrapper.build_sql()
    if conn is None:
        new_conn = data_source.get_reusable_connection()
        try:
            new_conn.acquire(operation_id=operation_id)
            new_conn.begin()
            row_affected, _ = data_source.get_executor().execute(new_conn, sql, args)
            new_conn.commit()
            return row_affected or 0
        finally:
            new_conn.release(operation_id=operation_id)
    row_affected, _ = data_source.get_executor().execute(conn, sql, args)
    return row_affected or 0



from typing import Any, Dict, List, Literal, Tuple, TypeVar

from ._insert_wrapper import InsertWrapper
from ._middlewares import before_insert_middlewares
from .mysql._mysql_data_source import MysqlDataSource
from .mysql._reusable_mysql_connection import ReusableMysqlConnection
from .utils.random_utils import generate_random_string

T = TypeVar("T", bound=Any)


def insert(
    wrapper: InsertWrapper[T],
    data: Dict[str, Any],
    duplicate_key_update: List[str] | Literal["all"] | None = None,
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
) -> Tuple[int, int]:

    if data_source is None:
        raise ValueError("data_source must be provided")

    operation_id = generate_random_string("D-", 10)

    for middleware in before_insert_middlewares:
        if callable(middleware):
            middleware(data)

    sql, args = wrapper.build_insert_sql(data, duplicate_key_update)
    if conn is None:
        new_conn = data_source.get_reusable_connection()
        try:
            new_conn.acquire(operation_id=operation_id)
            new_conn.begin()
            row_affected, last_row_id = data_source.get_executor().execute(new_conn, sql, args)
            new_conn.commit()
            return row_affected, last_row_id
        finally:
            new_conn.release(operation_id=operation_id)
    return data_source.get_executor().execute(conn, sql, args)


def insert_bulk(
    wrapper: InsertWrapper[T],
    data: List[Dict[str, Any]],
    duplicate_key_update: List[str] | Literal["all"] | None = None,
    conn: ReusableMysqlConnection | None = None,
    data_source: MysqlDataSource | None = None,
) -> int:

    if data_source is None:
        raise ValueError("data_source must be provided")

    operation_id = generate_random_string("D-", 10)

    for middleware in before_insert_middlewares:
        if callable(middleware):
            middleware(data)

    sql, args = wrapper.build_insert_bulk_sql(data, duplicate_key_update)
    if conn is None:
        new_conn = data_source.get_reusable_connection()
        try:
            new_conn.acquire(operation_id=operation_id)
            new_conn.begin()
            row_affected = data_source.get_executor().executemany(new_conn, sql, args)
            new_conn.commit()
            return row_affected
        finally:
            new_conn.release(operation_id=operation_id)
    return data_source.get_executor().executemany(conn, sql, args)



import threading
from typing import Callable

from loguru import logger
from pymysql import MySQLError
from pymysql.connections import Connection
from pymysql.cursors import DictCursor
from ..errors import ConnectionException

from .. import settings


class ReusableMysqlConnection:
    def __init__(self, data_source_id: str, create_connection: Callable[[], Connection]):
        self._data_source_id = data_source_id
        self._lock = threading.Lock()
        self._create_connection = create_connection
        self._conn: Connection | None = None
        self._active = False
        self._closed = False
        self._in_use = False  # 添加使用中标记

        self._stop_event = threading.Event()  # 添加停止事件
        # 其他初始化代码
        self._keep_alive_thread = threading.Thread(target=self._keep_alive_worker, daemon=True)
        self._keep_alive_thread.start()

    def is_locked(self) -> bool:
        """
        检查锁是否已经被占用。

        Returns:
            bool: 如果锁被占用，返回 True；否则返回 False。
        """
        return self._lock.locked()

    def acquire(self, timeout: float = 5, operation_id: str | None = None):
        if settings.enable_connection_lock_log:
            logger.debug(
                f"[{operation_id}] try to lock connection[{id(self._conn)}] with timeout {timeout} seconds."
            )
        if self._lock.acquire(timeout=timeout):
            try:
                self._in_use = True
                if not self._active:
                    self._conn = self._create_connection()
                    self._active = True
                    logger.info(f"[{self._data_source_id}] Connection created.")
                if settings.enable_connection_lock_log:
                    logger.debug(f"'[{operation_id}] Connection[{id(self._conn)}] locked.")
            except Exception as e:
                self._in_use = False  # 添加这行
                self._lock.release()  # 添加这行
                raise ConnectionException(f"Failed to create connection: {e}")
        else:
            raise ConnectionException(f"Failed to acquire connection within {timeout} seconds.")

    def release(self, operation_id: str | None = None):
        self._in_use = False  # 取消使用中标记
        self._lock.release()
        if settings.enable_connection_lock_log:
            logger.debug(f"[{operation_id}] Connection[{id(self._conn)}] released.")

    def _check_connection(self):
        if not self._in_use:
            raise ConnectionException(
                f"[{self._data_source_id}] Connection must be acquired before use."
            )
        if self._closed:
            raise ConnectionException(f"[{self._data_source_id}] Connection is closed.")
        if not self._active:
            raise ConnectionException(f"[{self._data_source_id}] Connection is not active.")

    def cursor(self) -> DictCursor:
        self._check_connection()
        try:
            if self._conn is None:
                raise ConnectionException(
                    f"[{self._data_source_id}] Connection is not initialized."
                )
            return self._conn.cursor()  # type: ignore
        except MySQLError as e:
            logger.error(
                f"[{self._data_source_id}] Connection[{id(self._conn)}] cursor creation failed: {e}"
            )
            try:
                self._conn = self._create_connection()
                logger.info(
                    f"[{self._data_source_id}] Connection[{id(self._conn)}] recreated after cursor failure."
                )
                return self._conn.cursor()  # type: ignore
            except Exception as recreate_error:
                logger.error(
                    f"[{self._data_source_id}] Failed to recreate connection: {recreate_error}"
                )
                self._active = False
                raise ConnectionException(f"Connection recreation failed: {recreate_error}")

    def begin(self):
        self._check_connection()
        try:
            if self._conn is None:
                raise ConnectionException(
                    f"[{self._data_source_id}] Connection is not initialized."
                )
            self._conn.begin()
        except MySQLError as e:
            logger.error(f"[{self._data_source_id}] Connection[{id(self._conn)}] begin failed: {e}")
            self._recreate_connection()
            raise ConnectionException(f"Transaction begin failed: {e}")

    def commit(self):
        self._check_connection()
        try:
            if self._conn is None:
                raise ConnectionException(
                    f"[{self._data_source_id}] Connection is not initialized."
                )
            self._conn.commit()
        except MySQLError as e:
            logger.error(
                f"[{self._data_source_id}] Connection[{id(self._conn)}] commit failed: {e}"
            )
            self._recreate_connection()
            raise ConnectionException(f"Transaction commit failed: {e}")

    def rollback(self):
        self._check_connection()
        try:
            self._conn.rollback()
        except MySQLError as e:
            logger.error(
                f"[{self._data_source_id}] Connection[{id(self._conn)}] rollback failed: {e}"
            )
            self._recreate_connection()
            raise ConnectionException(f"Transaction rollback failed: {e}")

    def _recreate_connection(self):
        try:
            old_conn_id = id(self._conn) if self._conn else None
            self._conn = self._create_connection()
            logger.info(
                f"[{self._data_source_id}] Connection[{old_conn_id}] -> [{id(self._conn)}] recreated."
            )
        except Exception as recreate_error:
            logger.error(
                f"[{self._data_source_id}] Failed to recreate connection: {recreate_error}"
            )
            self._active = False
            raise ConnectionException(f"Connection recreation failed: {recreate_error}")

    def close(self):
        if self._closed:
            return
        self._closed = True
        self._stop_event.set()  # 通知保活线程停止

        # 等待保活线程结束
        if hasattr(self, "_keep_alive_thread"):
            self._keep_alive_thread.join(timeout=5)

        with self._lock:
            try:
                if self._conn:
                    self._conn.close()
                    logger.info(f"[{self._data_source_id}] Connection[{id(self._conn)}] closed.")
            except MySQLError as e:
                logger.error(
                    f"[{self._data_source_id}] Connection[{id(self._conn)}] close failed: {e}"
                )
            finally:
                self._active = False

    def _keep_alive_worker(self):
        while not self._stop_event.wait(30):
            if self._closed:
                break
            if self._active and not self._in_use:  # 添加使用中检查
                if self._lock.acquire(timeout=1):  # 使用超时避免阻塞
                    try:
                        if self._conn and not self._in_use:  # 双重检查
                            self._conn.ping(reconnect=True)
                    except MySQLError as e:
                        logger.error(f"[{self._data_source_id}] ping failed: {e}")
                        self._recreate_connection()
                    finally:
                        self._lock.release()



from dataclasses import asdict
from typing import Any, Dict, List, Literal, Tuple, Type, TypeVar

from loguru import logger
from pymysql.cursors import DictCursor

from ._data_source_storage import DataSourceStorage
from ._delete import delete
from ._delete_wrapper import DeleteWrapper
from ._insert import insert, insert_bulk
from ._insert_wrapper import InsertWrapper
from ._query import find, find_dict, list as list_obj, list_dict, page as page_obj, page_dict
from ._query_wrapper import QueryWrapper
from ._update import update
from ._update_wrapper import UpdateWrapper
from .mysql import ReusableMysqlConnection
from .utils.random_utils import generate_random_string

T = TypeVar("T", bound=Any)


class Dorm:
    def __init__(self):
        self._init = False
        self._config_dict = None
        self._dss = DataSourceStorage()

        self._tx_id = None

    def is_initialized(self):
        return self._init

    def init(self, config_dict: Dict[str, Any]):
        self._config_dict = config_dict
        self._dss.load(config_dict)
        self._init = True
        logger.info("dorm initialized")

    def qw(self, cls: Type[T]) -> QueryWrapper[T]:
        return QueryWrapper[T](cls)

    def uw(self, cls: Type[T]) -> UpdateWrapper[T]:
        return UpdateWrapper[T](cls)

    def dw(self, cls: Type[T]) -> DeleteWrapper[T]:
        return DeleteWrapper[T](cls)

    def find(
        self,
        wrapper: QueryWrapper[T],
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> T | None:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        return find(wrapper, conn=conn, data_source=ds)

    def find_dict(
        self,
        wrapper: QueryWrapper[T],
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> Dict[str, Any] | None:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        return find_dict(wrapper, conn=conn, data_source=ds)

    def list(
        self,
        wrapper: QueryWrapper[T],
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> List[T]:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        return list_obj(wrapper, conn=conn, data_source=ds)

    def list_dict(
        self,
        wrapper: QueryWrapper[T],
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> List[Dict[str, Any]]:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        return list_dict(wrapper, conn=conn, data_source=ds)

    def page(
        self,
        wrapper: QueryWrapper[T],
        current: int,
        page_size: int,
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> Tuple[List[T], int]:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        return page_obj(wrapper, conn=conn, data_source=ds, current=current, page_size=page_size)

    def page_dict(
        self,
        wrapper: QueryWrapper[T],
        current: int,
        page_size: int,
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> Tuple[List[Dict[str, Any]], int]:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        return page_dict(wrapper, conn=conn, data_source=ds, current=current, page_size=page_size)

    def insert(
        self,
        cls: Type[T],
        data: Dict[str, Any] | T,
        duplicate_key_update: List[str] | Literal["all"] | None = None,
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> Tuple[int, int]:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        wrapper = InsertWrapper[T](cls)
        dict_data: Dict[str, Any] = data if isinstance(data, Dict) else asdict(data)
        return insert(wrapper, dict_data, duplicate_key_update, conn=conn, data_source=ds)

    def insert_bulk(
        self,
        cls: Type[T],
        data: List[Dict[str, Any]],
        duplicate_key_update: List[str] | Literal["all"] | None = None,
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> int:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")

        wrapper = InsertWrapper[T](cls)
        return insert_bulk(wrapper, data, duplicate_key_update, conn=conn, data_source=ds)

    def update(
        self,
        wrapper: UpdateWrapper[T],
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> int:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        return update(wrapper, conn=conn, data_source=ds)

    def delete(
        self,
        wrapper: DeleteWrapper[T],
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> int:
        ds = self._dss.get(data_source_id)
        if ds is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        return delete(wrapper, conn=conn, data_source=ds)

    def raw_query(
        self,
        sql: str,
        args: tuple[Any, ...],
        conn: ReusableMysqlConnection | None = None,
        data_source_id="default",
    ) -> List[Dict[str, Any]]:
        raw_query_id = generate_random_string("raw-query-", 10)

        sql = sql.replace("?", "%s")

        if conn is None:
            ds = self._dss.get(data_source_id)
            if ds is None:
                raise ValueError(f"Data source with ID '{data_source_id}' not found")
            new_conn = ds.get_reusable_connection()
            try:
                new_conn.acquire(operation_id=raw_query_id)
                new_conn.begin()
                cursor: DictCursor = new_conn.cursor()
                result = cursor.execute(sql, args)
                if result is None:
                    return []
                rows = cursor.fetchall()
                if rows is None:
                    return []
                return list(rows)
            finally:
                new_conn.release(operation_id=raw_query_id)
        else:
            cursor: DictCursor = conn.cursor()
            result = cursor.execute(sql, args)
            if result is None:
                return []
            rows = cursor.fetchall()
            if rows is None:
                return []
            return list(rows)

    def begin(self, data_source_id="default") -> ReusableMysqlConnection:
        self._tx_id = generate_random_string("tx-", 10)

        data_source = self._dss.get(data_source_id)
        if data_source is None:
            raise ValueError(f"Data source with ID '{data_source_id}' not found")
        conn = data_source.get_reusable_connection()
        conn.acquire(operation_id=self._tx_id)
        try:
            conn.begin()
            return conn
        except Exception as e:
            logger.error(f"Failed to begin transaction on data source '{data_source_id}': {e}")
            conn.release(operation_id=self._tx_id)
            raise e

    def commit(self, conn: ReusableMysqlConnection):
        if conn is None:
            raise RuntimeError("No connection to commit")
        try:
            conn.commit()
        except Exception as e:
            logger.error(f"Failed to commit transaction: {e}")
            raise e
        finally:
            conn.release(operation_id=self._tx_id)

    def rollback(self, conn: ReusableMysqlConnection):
        if conn is None:
            raise RuntimeError("No connection to rollback")
        try:
            conn.rollback()
        except Exception as e:
            logger.error(f"Failed to rollback transaction: {e}")
            raise e
        finally:
            conn.release(operation_id=self._tx_id)

    def add_data_source(
        self,
        data_source_id: str,
        dialect: str,
        host: str,
        port: int,
        user: str,
        password: str,
        database: str,
        **options: Any,
    ):
        self._dss.add_datasource(data_source_id, dialect, host, port, user, password, database, **options)

    def get_data_source(self, data_source_id="default"):
        return self._dss.get(data_source_id)


dorm = Dorm()



from ._delete_wrapper import DeleteWrapper
from ._dorm import dorm
from ._initializer import init
from ._insert_wrapper import InsertWrapper
from ._middlewares import use_insert_middleware, use_query_middleware
from ._query_wrapper import QueryWrapper
from ._update_wrapper import UpdateWrapper

__author__ = "melon"
__version__ = "0.10.3"

__all__ = [
    "init",
    "dorm",
    "use_query_middleware",
    "use_insert_middleware",
    "QueryWrapper",
    "DeleteWrapper",
    "UpdateWrapper",
    "InsertWrapper",
]



import setuptools

setuptools.setup(
    name="pydorm",
    version="0.10.3",
    description="A dynamic and lightweight Python orm framework",
    author="melon",
    packages=setuptools.find_packages(),
)
